#include "Common.h"
#include "HiewLua.h"
#include "Viewer.h"
#include "resource.h"

//------------------------------------------------------------------------
// Local data
//
static HIEWGATE_GETHEM2HEMGATE HiewLua;                         // gate to HiewLua plugin
static HIEWLUA_COMMAND HiewLuaCmd = {sizeof(HIEWLUA_COMMAND)};  // HiewLua never changes the cbSize member so its safe to initialize it only once

static int CALL_CONV hiew_setoffset(PLUASTATE L)
{
    OutOffset.LowPart = luaL_checkdword(L, 1);
    OutOffset.HighPart = luaL_optdword(L, 2, 0);
    OutFlags |= HEM_RETURN_SETOFFSET;
    return 0;
}

static int CALL_CONV hiew_setreload(PLUASTATE L)
{
    OutFlags |= HEM_RETURN_FILERELOAD;
    return 0;
}

static int CALL_CONV hiew_setmode(PLUASTATE L)
{
    int Mode = luaL_checkinteger(L, 1);
    switch(Mode)
    {
        case HEM_RETURN_MODE_TEXT:
        case HEM_RETURN_MODE_HEX:
        case HEM_RETURN_MODE_CODE:      break;

        default:
            lua_pushstring(L, "hiew.setmode(): mode must be hiew.TEXT, hiew.HEX or hiew.CODE");
            lua_error(L);
    }
    OutMode = Mode;
    OutFlags |= HEM_RETURN_SETMODE;
    return 0;
}

static int CALL_CONV hiew_exit(lua_State *L)
{
    if(!PostMessage(hDialogWnd, WM_COMMAND, IDC_EXITTOHIEW, 0))
    {
        lua_pushstring(L, "exit(): PostMessage() failed");
        lua_error(L);
    }
    lua_pushstring(L, "");
    lua_error(L);
    return 0;
}

static const luaL_Reg hiew_funcs[] =
{
    {"setoffset", hiew_setoffset},
    {"setreload", hiew_setreload},
    {"setmode", hiew_setmode},
    {"exit", hiew_exit},
    {0, 0}
};

static void __stdcall Register(PLUASTATE L)
{
    luaL_register(L, "hiew", hiew_funcs);       // this leaves the hiew table on the top of the stack
    
    lua_pushinteger(L, HEM_RETURN_MODE_TEXT);
    lua_setfield(L, -2, "TEXT");
    
    lua_pushinteger(L, HEM_RETURN_MODE_HEX);
    lua_setfield(L, -2, "HEX");
    
    lua_pushinteger(L, HEM_RETURN_MODE_CODE);
    lua_setfield(L, -2, "CODE");
    
    lua_pushstring(L, HiewFileName);
    lua_setfield(L, -2, "filename");

    lua_pushinteger(L, HiewFileLength.LowPart);
    lua_setfield(L, -2, "filesizelo");
    lua_pushinteger(L, HiewFileLength.HighPart);
    lua_setfield(L, -2, "filesizehi");

    lua_pushinteger(L, HiewCurrentOffset.LowPart);
    lua_setfield(L, -2, "offsetlo");
    lua_pushinteger(L, HiewCurrentOffset.HighPart);
    lua_setfield(L, -2, "offsethi");

    lua_pop(L, 1);
}

//------------------------------------------------------------------------
// Call HiewLua
//
static int CallHiewLua(DWORD Cmd)
{
    HiewLuaCmd.Command = Cmd;
    return HiewLua.Hem2HemGate(&HiewLuaCmd);
}

//------------------------------------------------------------------------
// Load and setup HiewLua
//
bool LoadHiewLua()
{
    LPCTSTR pErrMsg;
    /* get HiewLua entry point */
    if(HEM_OK == HiewGate_GetHem2HemGate(&HiewLua, (PHEM_BYTE)"HiewLua") && HiewLua.Hem2HemGate != 0)
    {
        /* set additional Lua functions we want to be registered after opening of Lua */
        HiewLuaCmd.pRegister = Register;
        CallHiewLua(HIEWLUA_REGISTER);

        /* open Lua interpreter */
        switch(CallHiewLua(HIEWLUA_OPEN))
        {
            case HEM_OK:
                HiewLuaCmd.pPrintText = PrintStdOut;
                CallHiewLua(HIEWLUA_SETSTDOUT);
                HiewLuaCmd.pPrintText = PrintStdErr;
                CallHiewLua(HIEWLUA_SETSTDERR);
                return true;

            case HIEWLUA_ERR_ALREADYOPEN:   pErrMsg = "Lua interpreter is already open";    break;
            case HIEWLUA_ERR_OPENFAILED:    pErrMsg = "Could not open Lua interpreter";     break;
            default:                        pErrMsg = "HiewLua: internal error";
        }
    }
    else pErrMsg = "Could not load HiewLua plugin";

    ShowError(pErrMsg);
    return false;
}

//------------------------------------------------------------------------
// Close Lua
//
void CloseLua()
{
    /* remove StdOut & StdErr */
    HiewLuaCmd.pPrintText = 0;
    CallHiewLua(HIEWLUA_SETSTDOUT);
    CallHiewLua(HIEWLUA_SETSTDERR);

    /* close Lua engine */
    CallHiewLua(HIEWLUA_CLOSE);
}

//------------------------------------------------------------------------
// Execute script or file
//
int ExecuteScript(LPCTSTR pScript, HWND hNotifyWnd, UINT NotifyMsg)
{
    HiewLuaCmd.StringScript.pName = pScript;
    HiewLuaCmd.StringScript.hEvent = 0;             // we don't need notifications through event
    HiewLuaCmd.StringScript.hWnd = hNotifyWnd;      // however, we want a message telling us that script execution is over
    HiewLuaCmd.StringScript.Msg = NotifyMsg;        // this is the message we want to receive
    HiewLuaCmd.StringScript.wParam = 0;             // we don't care
    HiewLuaCmd.StringScript.lParam = 0;             // we don't care

    return CallHiewLua(HIEWLUA_EXECSTRING);
}

int ExecuteFile(LPCTSTR pScriptFile, HWND hNotifyWnd, UINT NotifyMsg)
{
    HiewLuaCmd.FileScript.pName = pScriptFile;
    HiewLuaCmd.FileScript.hEvent = 0;               // we don't need notifications through event
    HiewLuaCmd.FileScript.hWnd = hNotifyWnd;        // however, we want a message telling us that script execution is over
    HiewLuaCmd.FileScript.Msg = NotifyMsg;          // this is the message we want to receive
    HiewLuaCmd.FileScript.wParam = 0;               // we don't care
    HiewLuaCmd.FileScript.lParam = 0;               // we don't care

    return CallHiewLua(HIEWLUA_EXECFILE);
}

//------------------------------------------------------------------------
// Terminate script
// Returns true if script was really terminated
//
bool TerminateScript()
{
    if(HIEWLUA_ERR_STILLRUNNING == CallHiewLua(HIEWLUA_GETSCRIPTEXITCODE))
    {
        CallHiewLua(HIEWLUA_ABORTSCRIPT);
        return true;
    }
    return false;
}